from typing import Any, Callable, Dict, Generator

from ..field_erros import FieldDigitError, FieldInvalidError, FieldTypeError
from ..validators.te_validator import TEValidator

__all__ = ["TE"]


AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class TE(str):
    __slots__ = ["number"]

    def __init__(self, number: str) -> None:
        self.number = number

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(type="string", format="te")

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_numbers
        yield cls.validate

    @classmethod
    def validate_type(cls, value: str) -> str:
        if not isinstance(value, str):
            raise FieldTypeError()
        return value

    @classmethod
    def validate_numbers(cls, value: str) -> str:
        if not value.isdigit():
            raise FieldDigitError()
        return value

    @classmethod
    def validate(cls, value: str) -> str:
        te = TEValidator(value)
        if not te.validate():
            raise FieldInvalidError()
        return value



import re

__all__ = ["CPFValidator"]


class CPFValidator:
    def __init__(self, cpf: str) -> None:
        self.cpf = cpf

    def validate_mask(self) -> bool:
        if len(self.cpf) == 14:
            if self.cpf[3:4] == "." and self.cpf[7:8] == "." and self.cpf[11:12] == "-":
                return True
        return False

    def validate(self) -> bool:
        cpf = re.sub("[^0-9]", "", str(self.cpf))

        if len(set(cpf)) == 1:
            return False

        if len(cpf) != 11:
            return False

        first_digit = self._validate_first_digit(cpf)
        second_digit = self._validate_second_digit(cpf)
        return cpf[9] == first_digit and cpf[10] == second_digit

    def _validate_first_digit(self, cpf: str) -> str:
        sum = 0

        for i in range(10, 1, -1):
            sum += int(cpf[10 - i]) * i

        sum = (sum * 10) % 11

        if sum == 10:
            sum = 0

        return str(sum)

    def _validate_second_digit(self, cpf: str) -> str:
        sum = 0

        for i in range(11, 1, -1):
            sum += int(cpf[11 - i]) * i

        sum = (sum * 10) % 11

        if sum == 10:
            sum = 0

        return str(sum)



from typing import Any, Callable, Dict, Generator

from ..field_erros import (
    FieldDigitError,
    FieldInvalidError,
    FieldMaskError,
    FieldTypeError,
)
from ..validators.cpf_validator import CPFValidator

__all__ = [
    "CPF",
    "CPFMask",
    "CPFDigits",
]

AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class CPFBase(str):
    __slots__ = ["number"]

    def __init__(self, number: str) -> None:
        self.number = number

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(type="string", format="cpf")

    @classmethod
    def validate_type(cls, value: str) -> str:
        if not isinstance(value, str):
            raise FieldTypeError()
        return value

    @classmethod
    def validate(cls, value: str) -> str:
        cpf = CPFValidator(value)
        if not cpf.validate():
            raise FieldInvalidError()
        return value


class CPF(CPFBase):
    """
    Accepts string of CPF with or without mask.

    Attributes:
        number (str): CPF number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate


class CPFMask(CPFBase):
    """
    Only Accepts string of CPF with mask.

    Attributes:
        number (str): CPF number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_mask
        yield cls.validate

    @classmethod
    def validate_mask(cls, value: str) -> str:
        cpf = CPFValidator(value)
        if not cpf.validate_mask():
            raise FieldMaskError()
        return value


class CPFDigits(CPFBase):
    """
    Only Accepts string of CPF with digits.

    Attributes:
        number (str): CPF number.
    """

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_numbers
        yield cls.validate

    @classmethod
    def validate_numbers(cls, value: str) -> str:
        if not value.isdigit():
            raise FieldDigitError()
        return value



try:
    from pydantic import PydanticTypeError, PydanticValueError
except ModuleNotFoundError:
    raise ModuleNotFoundError("Are you sure you installed pydantic")

__all__ = [
    "FieldTypeError",
    "FieldMaskError",
    "FieldDigitError",
    "FieldInvalidError",
]


class FieldTypeError(PydanticTypeError):
    code = "not_str"
    msg_template = "str type expected"


class FieldMaskError(PydanticValueError):
    code = "invalid_mask"
    msg_template = "invalid mask format"


class FieldDigitError(PydanticValueError):
    code = "not_digits"
    msg_template = "field only accept digits as string"


class FieldInvalidError(PydanticValueError):
    code = "invalid_data"
    msg_template = "invalid data"
